#include "myapplication.h"
#include "myinterpreter.h"

#include <iostream>

#include <QThread>
#include <QDebug>

#include "octave/octave.h"

Q_DECLARE_METATYPE(octave_value_list)

MyApplication::MyApplication(int argc, char **argv)
    : QApplication(argc, argv)
    , octave::application(argc, argv)
{
    qDebug() << "Initialising MyApplication.";

    m_interpreter = new MyInterpreter();
    m_interpreter->execute();
    workerThread = new QThread();

    // Register octave_value_list for signal/slot inter-thread comms.
    qRegisterMetaType<octave_value_list>();

    octave_map args;
    args.assign("currentFigure", Cell(octave_value(1)));
    parameters.append(args);
}

MyApplication::~MyApplication()
{
    if (m_interpreter->thread() == this->thread()) {
        // Closing app before m_interpreter moved to worker thread.
        delete m_interpreter;
    }
    workerThread->quit();
    workerThread->wait();
}

// Try and initiate the graphics stack, then move the interpreter
// to its own QThread.
void MyApplication::startInterpreter(void)
{
    qDebug() << "Try and execute MyApplication";

    qDebug() << "Graphics at start:-";
    m_interpreter->printGraphics();

    m_interpreter->registerGraphics();

    qDebug() << "Graphics after registerGraphics:-";
    m_interpreter->printGraphics();

    m_interpreter->moveToThread(workerThread);

    connect(workerThread, &QThread::finished, m_interpreter, &QObject::deleteLater);
    connect(this, &MyApplication::launchTestFigure, m_interpreter, &MyInterpreter::runTestFigure);

    workerThread->start();
    emit launchTestFigure(&parameters);
    return;
}
